@@ -689,8 +689,7 @@ class BaseMarionetteTestRunner(object):
                               docroot=docroot)
         self.httpd.start()
         self.marionette.baseurl = 'http://%s:%d/' % (host, self.httpd.httpd.server_port)
-        self.logger.info('running webserver on %s serving content from %s' % (
-            self.marionette.baseurl, docroot))
+        self.logger.info('running webserver on %s' % self.marionette.baseurl)
 
 
     def _build_kwargs(self):
